@@ -31,7 +31,17 @@ namespace Breeze
 {
 
     //____________________________________________________________________
-    void MdiWindowShadow::updateGeometry( void )
+    MdiWindowShadow::MdiWindowShadow( QWidget* parent, TileSet shadowTiles ):
+        QWidget( parent ),
+        _shadowTiles( shadowTiles )
+    {
+        setAttribute( Qt::WA_OpaquePaintEvent, false );
+        setAttribute( Qt::WA_TransparentForMouseEvents, true );
+        setFocusPolicy( Qt::NoFocus );
+    }
+
+    //____________________________________________________________________
+    void MdiWindowShadow::updateGeometry()
     {
         if( !_widget ) return;
 
@@ -81,7 +91,7 @@ namespace Breeze
     }
 
     //____________________________________________________________________
-    void MdiWindowShadow::updateZOrder( void )
+    void MdiWindowShadow::updateZOrder()
     { stackUnder( _widget ); }
 
     //____________________________________________________________________
@@ -117,12 +127,14 @@ namespace Breeze
         // store in set
         _registeredWidgets.insert( widget );
 
+        // create shadow immediatly if widget is already visible
         if( widget->isVisible() )
         {
             installShadow( widget );
             updateShadowGeometry( widget );
             updateShadowZOrder( widget );
         }
+
         widget->installEventFilter( this );
 
         // catch object destruction
@@ -188,7 +200,7 @@ namespace Breeze
     {
 
         // check object,
-        if( !object->parent() ) return 0L;
+        if( !object->parent() ) return nullptr;
 
         // find existing window shadows
         auto children = object->parent()->children();
@@ -198,7 +210,7 @@ namespace Breeze
             { if( shadow->widget() == object ) return shadow; }
         }
 
-        return 0L;
+        return nullptr;
 
     }
 
