@@ -60,15 +60,16 @@ namespace Breeze
         _atom( 0 )
         #endif
         #if BREEZE_HAVE_KWAYLAND
-        , _shadowManager( Q_NULLPTR )
-        , _shmPool( Q_NULLPTR )
+        , _shadowManager( nullptr )
+        , _shmPool( nullptr )
         #endif
     {
-        initializeWayland();
+        // delay till event dispatcher is running as Wayland is highly async
+        QMetaObject::invokeMethod(this, "initializeWayland", Qt::QueuedConnection);
     }
 
     //_______________________________________________________
-    ShadowHelper::~ShadowHelper( void )
+    ShadowHelper::~ShadowHelper()
     {
 
         #if BREEZE_HAVE_X11
@@ -89,17 +90,17 @@ namespace Breeze
         if( !connection ) {
             return;
         }
-        Registry *registry = new Registry( this );
+        auto registry = new Registry( connection );
         registry->create( connection );
         connect(registry, &Registry::interfacesAnnounced, this,
             [registry, this] {
                 const auto interface = registry->interface( Registry::Interface::Shadow );
                 if( interface.name != 0 ) {
-                    _shadowManager = registry->createShadowManager( interface.name, interface.version, this );
+                    _shadowManager = registry->createShadowManager( interface.name, interface.version, registry );
                 }
                 const auto shmInterface = registry->interface( Registry::Interface::Shm );
                 if( shmInterface.name != 0 ) {
-                    _shmPool = registry->createShmPool( shmInterface.name, shmInterface.version, this );
+                    _shmPool = registry->createShmPool( shmInterface.name, shmInterface.version, registry );
                 }
             }
         );
@@ -110,7 +111,7 @@ namespace Breeze
     }
 
     //______________________________________________
-    void ShadowHelper::reset( void )
+    void ShadowHelper::reset()
     {
         #if BREEZE_HAVE_X11
         if( Helper::isX11() )
@@ -155,7 +156,7 @@ namespace Breeze
     }
 
     //_______________________________________________________
-    void ShadowHelper::loadConfig( void )
+    void ShadowHelper::loadConfig()
     {
 
         // reset
@@ -170,23 +171,51 @@ namespace Breeze
     //_______________________________________________________
     bool ShadowHelper::eventFilter( QObject* object, QEvent* event )
     {
+        if( Helper::isWayland() )
+        {
 
-        // check event type
-        if( event->type() != QEvent::WinIdChange ) return false;
+            #if BREEZE_HAVE_KWAYLAND
+            QWidget* widget( static_cast<QWidget*>( object ) );
+            if( event->type() == QEvent::Paint )
+            {
 
-        // cast widget
-        QWidget* widget( static_cast<QWidget*>( object ) );
+                auto iter = _widgetSurfaces.constFind( widget );
+                if( iter == _widgetSurfaces.constEnd() )
+                {
+                    // install shadows and update winId
+                    installShadows( widget );
+                }
 
-        // install shadows and update winId
-        if( installShadows( widget ) )
-        { _widgets.insert( widget, widget->winId() ); }
+            } else if( event->type() == QEvent::Hide ) {
+
+                auto iter = _widgetSurfaces.find( widget );
+                if( iter != _widgetSurfaces.end() )
+                {
+                    _widgetSurfaces.erase( iter );
+                }
+
+            }
+            #endif
+
+        } else if( Helper::isX11() ) {
+
+            // check event type
+            if( event->type() != QEvent::WinIdChange ) return false;
+
+            // cast widget
+            QWidget* widget( static_cast<QWidget*>( object ) );
+
+            // install shadows and update winId
+            if( installShadows( widget ) )
+            { _widgets.insert( widget, widget->winId() ); }
+        }
 
         return false;
 
     }
 
     //_______________________________________________________
-    TileSet ShadowHelper::shadowTiles( void )
+    TileSet ShadowHelper::shadowTiles()
     {
         if( !_shadowTiles.isValid() )
         {
@@ -301,7 +330,7 @@ namespace Breeze
     }
 
     //______________________________________________
-    const QVector<quint32>& ShadowHelper::createPixmapHandles( void )
+    const QVector<quint32>& ShadowHelper::createPixmapHandles()
     {
 
         /**
@@ -412,7 +441,6 @@ namespace Breeze
         foreach( const quint32& value, pixmaps )
         { data.append( value ); }
 
-
         const QMargins margins = shadowMargins( widget );
         const int topSize = margins.top();
         const int bottomSize = margins.bottom();
@@ -463,6 +491,7 @@ namespace Breeze
         shadow->setOffsets( shadowMargins( widget ) );
         shadow->commit();
         s->commit( Surface::CommitFlag::None );
+        _widgetSurfaces.insert(widget, s);
 
         return true;
         #else
